#!/bin/bash

# This script interactively leads through the steps for starting Tweety, 
# the EIBnet/IP Tunnelling server (or eibd)
#
# Author: Bernhard Erb, Georg Neugschwandtner, Ulrich Krenn, Martin Kgler


function welcome() 
{
    if [ -f /var/run/tweety.pid -o -f `setup-eibd.sh -p -d /tmp/eibd` ]; then
	Xdialog --wrap --title "EIB/KNX Connectivity Setup" --msgbox "An EIBnet/IP Server is already running. Select OK to stop it." 10 41 || aborted
	"$0" -k
    fi

    Xdialog --left --wrap --title "Which interface?" \
	--menu "Welcome! The following dialogs will help you to	set up a KNXnet/IP (also known as EIBnet/IP) Tunnelling server. \n\nPlease select the type of interface you use. If you have a BCU attached via a serial cable, it probably is a 'BCU 1'. If 'BCU 1' does not work, try 'BCU 2' instead. " 24 60 2 \
        "BCU 1" "Use the PEI 16 protocol"\
        "BCU 2" "Use the PEI 10 (FT 1.2) protocol"\
        "USB" "Interface is connected via USB" 2> /tmp/menu.tmp.$$ || aborted
    
    proto=`cat /tmp/menu.tmp.$$`
    rm -f /tmp/menu.tmp.$$

##   This is now handled in a separate script
#
#    Xdialog --wrap --title "Start or recover?" \
#	--menu "You can now start the KNXnet/IP server \
#	using the chosen BCU. You may also correct the \
#	address table length of this BCU if necessary. \
#	If you are unsure what to do, start the EIBnet/IP \
#	server." 24 48 2 \
#	"1" "Start EIBnet/IP server"\
#	"2" "Reset table length" 2> /tmp/menu.tmp.$$ || aborted
#    
#    export start=`cat /tmp/menu.tmp.$$`
#    rm -f /tmp/menu.tmp.$$

    export start=1

    if [ "$proto" == "BCU 1" ]; then
	Xdialog --wrap --title "Which KNXnet/IP Server?" \
	    --menu "Two different KNXnet/IP servers for a BCU 1 are available. \nIf you are not sure which one to choose, try tweety." 24 48 2 \
	    "tweety" "A lightweight KNXnet/IP server"\
	    "eibd" "A multi purpose KNX network stack" 2> /tmp/menu.tmp.$$ || aborted
	      
	bcu1srv=`cat /tmp/menu.tmp.$$`
	rm -f /tmp/menu.tmp.$$
	  
	Xdialog --wrap --title "Which BCU 1 driver?" \
	    --menu "The PEI 16 protocol spoken by the BCU 1 needs to be handled by a special driver. Two different BCU 1 drivers are available. \nIf you are unsure what to do, try the first one." 24 48 2 \
	    "u" "BCU 1 driver eib-pei16-2.6" \
	    "m" "BCU 1 driver bcu1driver-0.2.6"  2> /tmp/menu.tmp.$$ || aborted
	      
	bcu1drv=`cat /tmp/menu.tmp.$$`
	rm -f /tmp/menu.tmp.$$
	
	if [ "$bcu1drver" == "m" ]; then
	    sudo /usr/share/knxlive/inseib m;
	else
	    sudo /usr/share/knxlive/inseib;
	fi
    fi

    if [ "$start" == "2" ]; then
	EIBDOPT="-R"
    else
	EIBDOPT="-i off -r off -t on"
    fi
    if [ "$proto" == "BCU 2" ]; then
	setup-eibd.sh -d /tmp/eibd -b bcu2 $EIBDOPT || aborted
	success
    fi
    if [ "$proto" == "USB" ]; then
	setup-eibd.sh -d /tmp/eibd -b usb $EIBDOPT || aborted
	success
    fi
    if [ "$proto" == "BCU 1" -a "$bcu1srv" == "eibd" ]; then
	setup-eibd.sh -d /tmp/eibd -b bcu1 $EIBDOPT || aborted
	success
    fi
    scan_ports
     
}


function scan_ports()
{
    #info box. "scanning ports".  uk

    Xdialog --title "Scanning ports" \
    --no-buttons --wrap \
    --infobox "Scanning for connected BCUs. This may take up to one minute. Please Wait!" 8 45 9999999 &
    scan_port_msg_pid=$!        #store process id for closing window (kill pid)



    # try to read the address table length to determine
    # where BCUs are connected

    tweety -d /dev/eib0 -r;
    com1=$?;
    tweety -d /dev/eib1 -r;
    com2=$?;
    tweety -d /dev/eib2 -r; 
    com3=$?;
    tweety -d /dev/eib3 -r;
    com4=$?;


    # build the menu entries for the next step

    bculist="";
    if [ $com1 = 0 ]; then
        bculist="/dev/eib0 [COM1-ttyS0] ";
    fi
    if [ $com2 = 0 ]; then
        bculist=$bculist"/dev/eib1 [COM2-ttyS1] ";
    fi
    if [ $com3 = 0 ]; then
        bculist=$bculist"/dev/eib2 [COM3-ttyS2] ";
    fi 
    if [ $com4 = 0 ]; then
        bculist=$bculist"/dev/eib3 [COM4-ttyS3] ";
    fi

    kill $scan_port_msg_pid	#kill info box		uk
    wait $scan_port_msg_pid	#wait till killed

    if [ "$bculist" != "" ]; then
        select_device $bculist
    else
        nothing_found
    fi
}

    
function select_device()
{
    Xdialog --title "Choose connection" \
        --wrap --menu "One or more BCUs were found. \nChoose the one you would like to work with:" 24 48 2 \
        $bculist 2> /tmp/menu.tmp.$$ || aborted
    
    choice=`cat /tmp/menu.tmp.$$`
    rm -f /tmp/menu.tmp.$$

    choose_action
}    


function choose_action()
{
    if [ $start = "1" ]; then
	read_table_size
    else
	write_table_length
    fi
}    


# This function is currently unused (but working)
function write_table_length()
{
    Xdialog --title "Write table length" \
        --wrap --inputbox "Enter the desired address table length in the field below. If you reached this dialog by accident and do not know what to do, choose 'Cancel'." \
        13 45 2> /tmp/inputbox.tmp.$$ || aborted

    input=`cat /tmp/inputbox.tmp.$$`
    rm -f /tmp/inputbox.tmp.$$

    tweety -d $choice -w $input || bailOut "Writing the address table length failed."
    success
}


function nothing_found()
{
    Xdialog --title "Nothing found" \
        --wrap --msgbox "Sorry, no BCUs were found. \
        Please check the connection and try again." 10 50
}


function read_table_size()
{
    tweety -d $choice -r | awk '/address table length is/ {printf $7}' > /tmp/eibtest.tmp.$$
    
    tablesize=`cat /tmp/eibtest.tmp.$$`
    
    rm -f /tmp/eibtest.tmp.$$

    Xdialog --left --wrap --title "Starting the server"\
    --msgbox "You are about to start the KNXnet/IP server. Should you be using a BCU which usually serves as something else but a serial interface (e.g., light switch), please consider the following: \n\nTo work properly, the server has to set the BCU address table length to 0. It will be automatically reset when the server is shut down. Should the BCU be disconnected before this is done, you will have to reset the table length manually to make the previous application work again. \nShould this be necessary, use the 'address table length changer' from the KNXLive! menu.\n\nYour current table length is "$tablesize". Please make a note of this value." 0 0 || aborted

    start_server $choice
}


function start_server()
{
    if /sbin/ip route get 224.0.23.12; then
	true
    else
	Xdialog --title "Network setup" --msgbox "You have no route for the Multicast addresses. A local route will be added for them."  10 41 || aborted
	sudo /sbin/route add default lo
    fi

    setsid tweety -d $1 </dev/null 2>/dev/null 1>/dev/null & 
    sudo echo $! > /var/run/tweety.pid 	

    sleep 2

    ps aux | grep tweety | grep /dev/eib
    
    result=$?

    if [ $result = 0 ];then
        Xdialog --title "Server started" \
        --wrap --infobox "The server started succesfully." 13 25 10000
    else
        bail_out "Could not start server"
    fi
    success
}


function success()
{
    Xdialog --title "Success" \
        --wrap --infobox "All setup operations terminated successfully. KNXnet/IP Tunnelling enabled software (for example, ETS or the Calimero Demo application available on this CD) can now access your KNX/EIB network via IP. \n\nEnjoy KNXLive!" 13 45 10000;
    exit 0
}


function bail_out()
{
    Xdialog --title "Error" \
        --msgbox "An error occurred: $1" 10 41 ;
        
    exit 1;
}


function aborted() 
{
    Xdialog --title "Aborted" \
        --msgbox "Setup aborted on user request."  10 41 ;
    exit 1
}

if [ "$1" == "-k" ]; then
    if [ -f /var/run/tweety.pid ]; then
	killall -SIGTERM tweety
	sleep 10
	killall -SIGKILL tweety
	sudo rm /var/run/tweety.pid
	if [ "$2" != "-q" ]; then
	    Xdialog --title "EIB" --msgbox "Server stopped" 10 41 ;
	fi
	exit 0
    fi
    setup-eibd.sh -k -d /tmp/eibd "$2"
    exit 0
fi

welcome
